#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h> /*F_CPU = 4000000; _delay_ms max = 65,535ms; _delay_us max = 192us */

#include "main.h"
#include "pin_macros.h" 

#define BCD_SIZEOF	5
#define LARGE_LIMIT_FI	145

#if defined (__AVR_ATmega8__)
#warning  ATmega8  4Meg
//4 Meg
	//1 1 The rising edge  //1 0 The falling edge of INT1 generates an interrupt request.
	//External Interrupt Request 0 Enable
#	define	EXT_ISR_ENABLE	GICR=_BV(INT0); \
							MCUCR=(1<<ISC01)|(0<<ISC00);
#	define	PRESCALER_RESET	SFIOR|=_BV(PSR2)
#	define	OCR	OCR2
#	define	TIMER2_COMPARE	TIMER2_COMP_vect
#	define	EN_ISR_TIMER2_COMPARE	TIMSK=_BV(OCIE2)
#	define	TCCR2_INIT	TCCR2=(1<<CS22)|(1<<CS21)|(0<<CS20);//  256
#	define	R_TIFR	TIFR
#	define	OCR1A_INIT	OCR1A=4000;// 1 
#else
#warning  ATmega48 1 meg
// 1 meg
	//1 1 The rising edge  //1 0 The falling edge of INT1 generates an interrupt request.
	//External Interrupt Request 0 Enable
#	define	EXT_ISR_ENABLE	EIMSK=_BV(INT0); \
							EICRA=(1<<ISC01)|(0<<ISC00);
#	define	PRESCALER_RESET	GTCCR|=_BV(PSRASY)	
#	define	OCR	OCR2A
#	define	TIMER2_COMPARE	TIMER2_COMPA_vect
#	define	EN_ISR_TIMER2_COMPARE	TIMSK2|=_BV(OCIE2A)
#	define	TCCR2_INIT	TCCR2B=(1<<CS22)|(0<<CS21)|(0<<CS20);//  64
#	define	R_TIFR	TIFR1
#	define	OCR1A_INIT	OCR1A=1000;// 1 
#endif

ISR(SIG_INTERRUPT0)//   .
{
	PRESCALER_RESET;//SFIOR|=_BV(PSR2);// Bit 0  PSR10: Prescaler Reset Timer/Counter1 and Timer/Counter0
	TCNT2=255;//!!!!
	OCR=TriacGate;
	FlagIsr=1;
	EnTriac=1;
}

ISR(TIMER2_COMPARE){//   
if (OCR<LARGE_LIMIT_FI){
	if (EnTriac) 
	ON(TRIAC);
	_delay_us(25);
	OFF(TRIAC);
	}
}

static void avr_init(void)
{
	//TCCR0=_BV(CS00);
	TCCR1B=(1<<CS10)|(1<<WGM12);// ctc mode
	
	//MCUCR=(1<<ISC01)|(0<<ISC00);//1 1 The rising edge  //1 0 The falling edge of INT1 generates an interrupt request.
	//GICR=_BV(INT0);//External Interrupt Request 0 Enable
	EXT_ISR_ENABLE;
	
	EN_ISR_TIMER2_COMPARE;//TIMSK=_BV(OCIE2);
	OCR1A_INIT;//OCR1A=4000;// 1 
	OCR=220;//156 ~= 10mSek, 220 -   , TCNT2     ,    ISR(SIG_INTERRUPT0)
	TCCR2_INIT;//TCCR2=(1<<CS22)|(1<<CS21)|(0<<CS20);//  256
	

	DRIVER(TRIAC,OUT);
	DRIVER(PIN_INT,PULLUP);
	
	DRIVER(COM_DIG_0,OUT);
	DRIVER(COM_DIG_1,OUT);
	DRIVER(COM_DIG_2,OUT);
	
	DRIVER(SEG_0,OUT);
	DRIVER(SEG_1,OUT);
	DRIVER(SEG_2,OUT);
	DRIVER(SEG_3,OUT);
	DRIVER(SEG_4,OUT);
	DRIVER(SEG_5,OUT);
	DRIVER(SEG_6,OUT);
	DRIVER(SEG_7,OUT);
	
	sei();

    return;
}

#if defined (__AVR_ATmega8__)
//-------------------EEPROM-------------------------
static inline unsigned char EEPROM_read(unsigned int uiAddress)
{
	while(EECR & (1<<EEWE));
	EEAR = uiAddress;
	EECR |= (1<<EERE);
	return EEDR;
}

static void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	if ( EEPROM_read(uiAddress)!=ucData){
		while(EECR & (1<<EEWE));
		cli();
		//asm("wdr");//  
		EEAR = uiAddress;
		EEDR = ucData;
		EECR |= (1<<EEMWE);
		EECR |= (1<<EEWE);
		sei();
		}
}
#else
static inline unsigned char EEPROM_read(unsigned int uiAddress)
{
	while(EECR & (1<<EEPE));
	EEAR = uiAddress;
	EECR |= (1<<EERE);
	return EEDR;
}
static void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	if ( EEPROM_read(uiAddress)!=ucData){
		while(EECR & (1<<EEPE));
		cli();
		asm("wdr");//  
		EEAR = uiAddress;
		EEDR = ucData;
		EECR |= (1<<EEMPE);
		EECR |= (1<<EEPE);
		sei();
		}
}
#endif

static void save_struct_eep(void)
{
	uint8_t * a= (uint8_t *)&Setting;
	Setting.crc=0;
		for(uint16_t j=0, i=EEP_ADR; j<sizeof(Setting); i++, j++, a++)
			{
			if (j<sizeof(Setting)-2) Setting.crc+=*a;
			EEPROM_write( i, *a);
			}
		eeprom_eer=0;
}

static void read_struct_eep(void)
{
	uint16_t crc=0;
	uint8_t *a= (uint8_t *)&Setting;
	for(uint16_t j=0, i=EEP_ADR; j<sizeof(Setting); i++, j++, a++)
		{*a=EEPROM_read(i);
		if (j<sizeof(Setting)-2) crc+=*a;
		}
	if ( crc!= Setting.crc ) {
		eeprom_eer=1;
		Setting.IncPower=1;
		}
		else
		{
		LargeLimitReg=100;
		if (Setting.RegimFi) //if Fi
			if (Setting.FiNumView) LargeLimitReg=LARGE_LIMIT_FI;// if Num
		}
}

static void led_off(void)
{
	// 
	OFF(COM_DIG_0);
	OFF(COM_DIG_1);
	OFF(COM_DIG_2);
	
	OFF(SEG_0);
	OFF(SEG_1);
	OFF(SEG_2);
	OFF(SEG_3);
	OFF(SEG_4);
	OFF(SEG_5);
	OFF(SEG_6);
	OFF(SEG_7);
}

//-------------------------------------
static void led_on(void)
{
	if ( led_buffer[znmesto] & _BV(0) ) ON(SEG_0);
	if ( led_buffer[znmesto] & _BV(1) ) ON(SEG_1);
	if ( led_buffer[znmesto] & _BV(2) ) ON(SEG_2);
	if ( led_buffer[znmesto] & _BV(3) ) ON(SEG_3);
	if ( led_buffer[znmesto] & _BV(4) ) ON(SEG_4);
	if ( led_buffer[znmesto] & _BV(5) ) ON(SEG_5);
	if ( led_buffer[znmesto] & _BV(6) ) ON(SEG_6);
	if ( led_buffer[znmesto] & _BV(7) ) ON(SEG_7);

	if ( znmesto==0 )	ON(COM_DIG_0);
	else
	if ( znmesto==1 )	ON(COM_DIG_1);
	else
	if ( znmesto==2 )	ON(COM_DIG_2);
}

//-------------------------------------
static void skan_key(void){
// 4mSek
	//Key=KEY_VOID;

#ifdef KATOD
	PORT_BUTTON |= BTNS_PULLUP;
#else
	PORT_BUTTON &= ~COM_BTN; 
#endif
	DDR_BUTTON &= ~BTNS_PULLUP;
	_delay_us(15);
	uint8_t pin=PIN_BUTTON;
#ifdef KATOD
	PORT_BUTTON &= ~BTNS_PULLUP;
#else
	PORT_BUTTON |= COM_BTN;
#endif
	DDR_BUTTON |= BTNS_PULLUP;
	
//--------------------------------------
#define KEY_PRESS(Key)	((pin & (Key))==0)
//---------------------------------------------------------------------------------------	
	//static uint8_t antdr, count_press, povtor;
	if ( KEY_PRESS(BTN_PLUS) || KEY_PRESS(BTN_MINUS) || KEY_PRESS(BTN_SET) ){//     
		if ( antdr>=pause_after_press[count_press] ) antdr=0;
		
		if (++antdr==3){
			//if ( KEY_PRESS(BTN_PLUS ) && KEY_PRESS(BTN_MINUS) )
			//	Key=KEY_OBE;
			//else
			if ( KEY_PRESS(BTN_PLUS) )
				Key=KEY_PLUS;
			if ( KEY_PRESS(BTN_MINUS ) )
				Key|=KEY_MINUS;
			if ( KEY_PRESS(BTN_SET ) ){
				Key|=KEY_SET;
				ViewReg=10;
				Flag|=VIEWREG;
				}
				
			if (count_press<sizeof(pause_after_press)-1) count_press++;
			if (++povtor>=sizeof(pause_after_press)+20) Inc=3;
			TimeState=200;
			}
		}
		else{
		Key=KEY_VOID;
		antdr=count_press=povtor=0;
		Inc=1;
		}
}

void inc_dec_param(uint8_t *p, uint8_t limit, uint8_t inc){	
	if (Key==KEY_PLUS){
		if ( ((*p)+inc)<=limit ) (*p)+=inc; else (*p)=0;
		}
	if (Key==KEY_MINUS){
		if ( ((*p)-inc)>=0 ) (*p)-=inc; else (*p)=limit;
		}
}

//-------------------------------------
void key_action(void){

	if (Key) {//     ,    ,    .   
		TimeOff=0;
		if (Flag & MIGANIE) {
			Flag &= ~MIGANIE;
			Blank=0;
			Key=0;
			return;
			}
		}
	
	if (regim==0){
		if (Key&KEY_SET) 
			regim=ST_INPUT_REG;
			else
			inc_dec_param(&PowerTmp, LargeLimitReg, Inc+(Setting.IncPower-1));
		Key=0;
		return;
		}
	
	if (regim==ST_INPUT_REG){
		if (povtor>0){// povtor -   ( )
			if (povtor>2) {
				regim=ST_INPUT_TIP_REG; 
				Var=ST_TIP_REG;
				}
			if ( (Key&(KEY_SET|KEY_MINUS)) == (KEY_SET|KEY_MINUS) ) {
				PowerTmp=0;
				regim=ST_INPUT_TIP_REG; 
				Var=0;
				}
			if ( (Key&(KEY_SET|KEY_PLUS)) == (KEY_SET|KEY_PLUS) ) {
				PowerTmp=LargeLimitReg;
				regim=ST_INPUT_TIP_REG; 
				Var=0;
				}
			}
			else
			{ 
			if (Key==KEY_VOID) regim=ST_PRESET;
			}
		return;
		}
	
	if (regim==ST_INPUT_TIP_REG){
		if (Key==KEY_VOID) {
			Flag &= ~VIEWREG;
			regim=Var;
			} 
		return;
		}
	
	if (regim==ST_TIP_REG){
		inc_dec_param(&Setting.RegimFi, 1, 1);
		if (Key==KEY_SET){ 
			StepPreset=NUM_PRESET-1;//  ,           1- 
			PowerTmp=0;//  
			save_struct_eep();
			if (Setting.RegimFi) regim=ST_SET_VIEW_FI;
				else  {Var=0; regim=ST_SETTING_PRESET;}
			}
		Key=0;
		return;
		}
	
	if (regim==ST_SET_VIEW_FI){
		inc_dec_param(&Setting.FiNumView, 1, 1);
		if (Key==KEY_SET) {
			save_struct_eep(); 
			Var=0; 
			regim=ST_SETTING_PRESET;
			}
		Key=0;
		return;
		}
	
	if ( Key & (KEY_PLUS|KEY_MINUS))//  ()   
		if (Flag & VIEWREG) {
			Flag &= ~VIEWREG;
			Key=0;
			return;
			}
			
	if (regim==ST_SETTING_PRESET){
		uint8_t P=0;
		LargeLimitReg=100; 
		if (Setting.RegimFi){ //if Fi
			if (Setting.FiNumView){ 
				LargeLimitReg=LARGE_LIMIT_FI;
				P=NUM_PRESET;
				} 
				else 
				if (PowerTmp>100) PowerTmp=0;
			}
			else
			if (PowerTmp>100) PowerTmp=0;

		inc_dec_param(&Setting.Preset[Var+P], LargeLimitReg, Inc);
		if (Key==KEY_SET) {
			if (++Var>=NUM_PRESET-1){
				Setting.Preset[NUM_PRESET-1]=Setting.Preset[NUM_PRESET*2-1]=0;//   Preset,  OFF
				regim=ST_INC_POWER;
				}
			save_struct_eep();
			}
		Key=0;
		return;
		}
	if (regim==ST_INC_POWER){
		inc_dec_param(&Setting.IncPower, 20, 1);
		if (Setting.IncPower==0) Setting.IncPower=1;
		if (Key==KEY_SET) {
			save_struct_eep(); 
			regim=ST_TIM;
			}
		Key=0;
		return;
		}
	if (regim==ST_TIM){
		inc_dec_param(&Setting.Tim, 1, 1);
		if (Setting.Tim==0) Blank=0;
		if (Key==KEY_SET) {
			save_struct_eep(); 
			regim=ST_END;
			}
		Key=0;
		return;
		}
	
	if (regim==ST_END){
		if ( Key || ((Flag&VIEWREG)==0) ) {
			Flag &= ~VIEWREG;
			regim=0;
			Key=0;
			}
		return;
		}
	//------------PRESET-------------------------
	if (regim==ST_PRESET){
		if (++StepPreset>NUM_PRESET-1){
			StepPreset=0;
			}
		uint8_t P=0;
		if (Setting.RegimFi) //if Fi
			if (Setting.FiNumView){ 
				P=NUM_PRESET;
				}
		PowerTmp=Setting.Preset[StepPreset+P];
		Flag &= ~VIEWREG;
		regim=0;
		Key=0;
		return;
		}
}


void itoa2(int16_t Val, uint8_t * Buf)
{
const uint16_t step[BCD_SIZEOF]={10000,1000,100,10,1};
#if BCD_SIZEOF != 5 
#warning *** BCD_SIZEOF!=sizeof(step)! ***
#endif
int16_t Subtract, Value;
uint8_t i, Dig;

	Value=Val;
	for (i=0; i<BCD_SIZEOF; i++)
	{
	Subtract=step[i];
	Dig=0;
	while(Value >= Subtract)
		{
		Dig++;
		Value-=Subtract;
		}
	Buf[i]=Dig;
	}
}

void data_led(void){
uint8_t buf[BCD_SIZEOF];
	led_buffer[0]=0;
	led_buffer[1]=0;
	led_buffer[2]=0;
	
	if ((regim==0)||(regim==ST_PRESET)||(regim==ST_INPUT_REG)){
		if (eeprom_eer){
			led_buffer[0]=font[f_E];
			led_buffer[1]=font[f_E];
			led_buffer[2]=font[f_P];
			}
			else
			{
			if (EnTriac==0){
				led_buffer[0]=font[f_n];
				led_buffer[1]=font[f_o];
				led_buffer[2]=font[f_C]|sH;
				}
			else
			if (PowerTmp==0){
				led_buffer[0]=sD;//font[0];
				led_buffer[1]=sD;//font[f_F];
				led_buffer[2]=sD;//font[f_F];
				}
				else{
				if (Blank==0)
				{
				itoa2(PowerTmp, buf);
				led_buffer[0]=font[buf[2]];
				led_buffer[1]=font[buf[3]];
				led_buffer[2]=font[buf[4]];
				}
				}
			}
		return;
		}
	
		
	if (regim==ST_INPUT_TIP_REG){
		if ( (Key&(KEY_SET|KEY_MINUS)) == (KEY_SET|KEY_MINUS) ) {
			led_buffer[0]=font[0];
			led_buffer[1]=font[f_F];
			led_buffer[2]=font[f_F];
			}
		else
		if ( (Key&(KEY_SET|KEY_PLUS)) == (KEY_SET|KEY_PLUS) ) {
			led_buffer[0]=0;
			led_buffer[1]=font[f_o];
			led_buffer[2]=font[f_n];
			}
		else{
			led_buffer[0]=font[f_P];
			led_buffer[1]=font[f_E];
			led_buffer[2]=font[f_G];
			}
		return;
		}
	if (regim==ST_TIP_REG){
		if (Setting.RegimFi){
			led_buffer[1]=font[f_F];
			led_buffer[2]=font[f_I];
			}
			else{
			led_buffer[0]=font[f_P];
			led_buffer[1]=font[f_A];
			led_buffer[2]=font[f_U];
			}	
		return;
		}	
	if (regim==ST_SET_VIEW_FI){
		if (Setting.FiNumView){
			led_buffer[0]=font[4];
			led_buffer[1]=font[f_U];
			led_buffer[2]=font[f_C];//
			}
			else{
			led_buffer[0]=font[f_N];
			led_buffer[1]=font[f_P];
			led_buffer[2]=font[f_U]|sH;//
			}	
		return;
		}
	
		
	if (regim==ST_SETTING_PRESET)//
		{
		if (Flag & VIEWREG)
			{
			led_buffer[0]=sG;
			led_buffer[1]=font[Var+1];
			led_buffer[2]=sG;
			}
			else{
			uint8_t P;
			if ((Setting.RegimFi) && (Setting.FiNumView)) P=NUM_PRESET; else P=0;
			itoa2(Setting.Preset[Var+P], buf);
			led_buffer[0]=font[buf[2]];
			led_buffer[1]=font[buf[3]];
			led_buffer[2]=font[buf[4]];
			}
		return;
		}
		
	if (regim==ST_INC_POWER)//
		{
		if (Flag & VIEWREG)
			{
			led_buffer[0]=font[f_i];
			led_buffer[1]=font[f_n];
			led_buffer[2]=font[f_c];
			}
			else{
			itoa2(Setting.IncPower, buf);
			led_buffer[0]=font[buf[2]];
			led_buffer[1]=font[buf[3]];
			led_buffer[2]=font[buf[4]];
			}
		return;
		}
		
	if (regim==ST_TIM)//
		{
		if (Flag & VIEWREG)
			{
			led_buffer[0]=sD;//font[f_i];
			led_buffer[1]=font[f_t];
			led_buffer[2]=sD;//font[f_c];
			}
			else{
			if (Setting.Tim){
				//led_buffer[0]=0;//font[f_i];
				led_buffer[1]=font[f_o];
				led_buffer[2]=font[f_n];
				}
				else{
				led_buffer[0]=font[0];
				led_buffer[1]=font[f_F];
				led_buffer[2]=font[f_F];
				}
			}
		return;
		}
		
	if (regim==ST_END)//
		{
		led_buffer[0]=font[f_E];
		led_buffer[1]=font[f_n];
		led_buffer[2]=font[f_d];
		return;
		}
}

int main(void)
{
uint8_t TimeBlank=1;//Tim10msek=10,
uint8_t Tim100msek=10, TimLed=4,ChangePower=0;

avr_init();
read_struct_eep();
TriacGate=LARGE_LIMIT_FI;//


while(1)
{


if (Setting.RegimFi==0)
{//   
	if (FlagIsr)
	{
	FlagIsr=0;
	//u8 Power; //  
	//Power = MyPower; //   
	#define MAXPOW 100; //, . . 
	static uint8_t Phase = MAXPOW;//,  1 
	static int8_t Delta;

	//     0:
	Delta = Power + Delta;
	if (Delta < 0)
		{
		TriacGate=LARGE_LIMIT_FI;//TriacOff();
		}
		else
		{
		TriacGate=0;//TriacOn();
		Delta = Delta - MAXPOW;
		/*
			ON(TRIAC);
			_delay_us(10);
			OFF(TRIAC);*/
		}
	Phase = Phase - 1;
	if (Phase == 0)
		{
		Phase = MAXPOW;
		Delta = - (Phase / 2);
		}
	
	if (Power==0) {	TriacGate=LARGE_LIMIT_FI;}
	}
}

if (R_TIFR & _BV(OCF1A)) // 0.001 sek
	{
	R_TIFR = _BV(OCF1A);

	
	if ( --TimLed==0 )// 
		{
		TimLed=4;
		led_off();//  
		//asm("wdr");//  
		skan_key();
		key_action();
		led_on();// . 
		if (++znmesto>=3)
			{
			znmesto=0x00;
			data_led();
			}
		}
	
	if (ChangePower!=PowerTmp){
		ChangePower=PowerTmp;
		//TimeOff=0;
		if (Setting.RegimFi) {// 
			if (Setting.FiNumView==0){//   
				TriacGate=LARGE_LIMIT_FI-((LARGE_LIMIT_FI)*PowerTmp)/100;
				}
				else{
				TriacGate=LARGE_LIMIT_FI-PowerTmp;
				}
			}
			else{
			Power=PowerTmp;//   
			}
		}
	{uint8_t tcnt2=TCNT2;
	if ( tcnt2>LARGE_LIMIT_FI+30){
		if ( tcnt2!=255)
			EnTriac=0;
		}
	}
		//if (Setting.RegimFi) TriacGate=Power;
	//----------=-------------------0.1-Sek---------------------------------------
	if ( --Tim100msek==0 )	//0.1sek
		{
		Tim100msek=100;
		TimeState--;
		if (TimeState==0) regim=0;
		if (--ViewReg==0) Flag&=~VIEWREG;
		
		if (PowerTmp==0) StepPreset=NUM_PRESET-1;//  ,           1- 
		
		if (Setting.Tim)
			{
			//-------__------------
			if (PowerTmp)
			{
			if (++TimeOff>TIME_POWER_OFF_ALARM){
				Flag|=MIGANIE;
				if (TimeOff>TIME_POWER_OFF) PowerTmp=0;//.    if (ChangePower!=PowerTmp)
				if (--TimeBlank==0)
					{
					if ( Blank )
						{//   
						Blank=0;
						TimeBlank=5;//0.5sek
						}
					else
						{
						Blank=1;
						TimeBlank=2;
						}
					}
				}
			}
			else{ 
			Blank=0;
			TimeBlank=1;
			}
			}
		}	
	
	}

}
    
}


